(ns serialize-test
    (:import (java.io ByteArrayOutputStream
                      ObjectOutputStream
                      ByteArrayInputStream
                      ObjectInputStream)))

(defn serialize
  "Serializes a single object, returning a byte array."
  [v]
  (with-open [bout (ByteArrayOutputStream.)
              oos (ObjectOutputStream. bout)]
    (.writeObject oos v)
    (.flush oos)
    (.toByteArray bout)))

(defn deserialize
  "Deserializes and returns a single object from the given byte array."
  [bytes]
  (with-open [ois (-> bytes ByteArrayInputStream. ObjectInputStream.)]
    (.readObject ois)))

(defn slurp-bytes
  "Slurp the bytes from a slurpable thing"
  [x]
  (with-open [out (java.io.ByteArrayOutputStream.)]
    (clojure.java.io/copy (clojure.java.io/input-stream x) out)
    (.toByteArray out)))

(defn dump-bytes
  "Dump data as bytes to a given file"
  [f x]
  (clojure.java.io/copy x (java.io.File. f)))

*clojure-version*

(def f "seq-1-8-master.dump")
(def x {'(1) 1})
(dump-bytes f (serialize x))
(deserialize (slurp-bytes f))

(hash x)
(hash (deserialize (serialize x)))
(hash (deserialize (slurp-bytes f)))
(= x (deserialize (slurp-bytes f)))
(= (deserialize (slurp-bytes f)) x)

